/**
 * hello_library.h
 *
 * This is the basic hello library interface class.
 *
 */


// ======== MB_HelloWorld =============================================
/**
 *	@brief Basic interface class.
 *
 * This is an example of how to create an interface file.  Note that
 * since this is in a header file, it is implied that any other code
 * needing a MB_HelloWorld interface are able to get to it.
 *
 * This is an example of some formulas
 *   The distance between \f$(x_1,y_1)\f$ and \f$(x_2,y_2)\f$ is
 * \f$\sqrt{(x_2-x_1)^2+(y_2-y_1)^2}\f$.
 *
 */

class MB_HelloWorld
{
public:

	/**
	 * @brief Main method example.
	 *
	 * This main method will print out "Hello World" using std::cout.
	 *
	 * @return the return value is probably 0.
	 */

	static int Main();

	/**
	 * @brief ExampEmum is an example of an enum
	 *
	 * An enum is an important way of describing constant integer type
	 * values.  If corresponding values are not picked, they are generated
	 * by the compiler. C++ allows you to create multiple enums with the
	 * same value.
	 */

	enum ExampEmum
	{
		/**
		 * ValuePicked is the value 2 and see how it is declared
		 */

		ValuePicked = 2,

		/**
		 * The value of ValueAuto is given by the compiler and it will
		 * be different to ValuePicked.
		 * @see MB_HelloWorld::ValuePicked
		 */

		ValueAuto,

		/**
		 * The value of PickAgain is 2 and notice how we now have 2
		 * values in the enum with the same value.
		 */

		PickAgain = 2
	};

	/**
	 * @brief Demonstrate a more complex definition.
	 *
	 * MemberFunction does not really exist.  If you try to call it, it will
	 * have a link time error.  The purpose is only to show what it looks like
	 * to document a more complex function.
	 *
	 * @param intp is a pointer to an array of integers
	 * @param foo is another parameter to a string
	 * @return You can describe the return value here.
	 */
	
	int MemberFunction(
		int	* intp,
		char * foo /** This shows how you can describe a foo here too */
	);
};


// ======== MB_InheritExample =============================================
/**
 *	@brief Example of inheritance
 *
 * The purpose of this class is to be an example of how doxygen shoes
 * inheritance.
 * 
 */

template <typename T>
class MB_InheritExample : public MB_HelloWorld
{

	/**
	 * @brief m_value is just some value !
	 */

	T				m_value;
	
	
};
